@@ -14,6 +14,10 @@
 #include "ADM_image.h"
 #define ADM_INVALID_CACHE 0xffff
 #define ADM_IN_USE_CACHE  0xfffe
+
+#define EDITOR_CACHE_MIN_SIZE 8
+#define EDITOR_CACHE_MAX_SIZE 16
+
 typedef struct cacheElem
 {
 	ADMImage *image;
