@@ -191,11 +191,8 @@ uint32_t seg;
     uint64_t to=segTime+s->_refStartTimeUs;
     if(false==seektoTime(s->_reference,to))
     {
-        if(false==seektoTime(s->_reference,to)) // try twice because the first attempt may fail with VDPAU
-        {
-            ADM_warning("Cannot seek to beginning of segment %" PRIu32" at  %" PRIu64" ms\n",seg,to/1000);
-            return false;
-        }
+        ADM_warning("Cannot seek to beginning of segment %" PRIu32" at %s\n",seg,ADM_us2plain(to));
+        return false;
     }
     _currentSegment=seg;
     int64_t newTime=(int64_t)v->lastDecodedPts+(int64_t)s->_startTimeUs-(int64_t)s->_refStartTimeUs;
@@ -234,10 +231,8 @@ uint64_t tail;
                 goto np_nextSeg;
         }else 
         { // Not in cache, decode next one
-          // but don't go past the end of the last segment
-            uint64_t maxPts=0;
-            if(_currentSegment+1 == _segments.getNbSegments())
-                maxPts=seg->_refStartTimeUs+seg->_durationUs;
+          // but don't go past the end of the segment
+            uint64_t maxPts=seg->_refStartTimeUs+seg->_durationUs;
             if(false== nextPictureInternal(seg->_reference,image,maxPts))
             {
                 if(flags & 2)
