@@ -18,10 +18,7 @@
 #include "ADM_default.h"
 #include "A_functions.h"
 #include "ADM_audioFilterInterface.h"
-#include "audioEncoderApi.h"
-#include "ADM_muxerProto.h"
 #include "GUI_ui.h"
-#include "ADM_coreVideoFilterFunc.h"
 
 #include <fcntl.h>
 #include <errno.h>
@@ -33,15 +30,14 @@
 #include "DIA_coreToolkit.h"
 #include "prefs.h"
 
-//#include "ADM_outputfmt.h"
 #include "ADM_edPtsDts.h"
 #include "ADM_vidMisc.h"
 #include "ADM_confCouple.h"
 #include "ADM_videoFilters.h"
-#include "ADM_videoEncoderApi.h"
-#include "ADM_videoFilterApi.h"
 #include "ADM_coreVideoFilterFunc.h"
+#include "ADM_coreVideoEncoder.h"
 #include "ADM_coreDemuxer.h"
+#include "ADM_aacinfo.h"
 
 //#define TEST_MPEG2DEC
 /**
@@ -62,9 +58,6 @@ ADM_Composer::ADM_Composer (void)
   markerAPts = 0;
   markerBPts = 0;
   stats.reset();
-
-    totalExtraDelay=0;
-    desyncScore=0;
 }
 /**
 	Remap 1:1 video to segments
@@ -153,6 +146,244 @@ bool ADM_Composer::setProjectName(const std::string &pj)
     currentProjectName=pj;
     return true;
 }
+
+/**
+    \fn checkStdTimeBase
+    \brief Try to replace timebase denominator with a standard value.
+*/
+static bool checkStdTimeBase(uint32_t &num, uint32_t &den)
+{
+    if(!num || !den)
+        return false;
+    if(den>num && !(den%num))
+    {
+        switch(den/num)
+        {
+            case 24: case 25: case 30: case 50: case 60:
+                den/=num;
+                num=1000;
+                den*=1000;
+                return true;
+            default:break;
+        }
+    }
+#define MAX_STD_CLOCK_FREQ 90000
+    switch(den)
+    {
+        case 24000: case 25000: case 30000: case 50000:
+            return true; // nothing to do
+        case 48000:
+            if(num>1 && !(num&1))
+            {
+                num>>=1;
+                den>>=1;
+                return true;
+            }
+            break;
+        case 90000:
+            if(num>=18 && !(num%18))
+            {
+                num/=18;
+                num*=5;
+                den=25000;
+                break;
+            }
+            if(num>=3 && !(num%3))
+            {
+                num/=3;
+                den=30000;
+            }
+            break;
+        case 180000:
+            if(num>1 && !(num&1))
+            {
+                num>>=1;
+                den=90000;
+                return checkStdTimeBase(num,den);
+            }
+            break;
+        case 10000000:
+            {
+                den/=10;
+                num/=10;
+                if(!num) num=1;
+                return checkStdTimeBase(num,den);
+            }
+            break;
+        case 1000000:
+            {
+                int n,d;
+                usSecondsToFrac(num,&n,&d,MAX_STD_CLOCK_FREQ);
+                num=n;
+                den=d;
+            }
+            break;
+        case 600: // libavformat doesn't like low timebase clocks
+            num*=50;
+            den=30000;
+            break;
+        case 1200:
+            num*=25;
+            den=30000;
+            return true;
+        case 2400:
+            num*=10;
+            den=24000;
+            return true;
+        default: break;
+    }
+    /* some old AVI samples clock insanely high */
+    if(den>=1000000)
+    {
+        double us=num;
+        us*=1000.;
+        us/=den;
+        us*=1000.;
+        us+=0.49;
+        int n,d;
+        usSecondsToFrac((uint64_t)us,&n,&d,MAX_STD_CLOCK_FREQ);
+        num=n;
+        den=d;
+    }
+    return true;
+}
+
+/**
+    \fn getTimeBase
+*/
+bool ADM_Composer::getTimeBase(uint32_t *scale, uint32_t *rate, bool copyMode)
+{
+    /* Using MPEG timescale as fallback */
+    *scale=1;
+    *rate=MAX_STD_CLOCK_FREQ;
+    /* Which segments does the selection span? */
+    uint64_t segTime,start,end;
+    start=getMarkerAPts();
+    end=getMarkerBPts();
+    if(start>end)
+    {
+        uint64_t tmp=end;
+        end=start;
+        start=tmp;
+    }
+    uint32_t firstSeg,lastSeg;
+    if(false==_segments.convertLinearTimeToSeg(start,&firstSeg,&segTime))
+        return false;
+    if(false==_segments.convertLinearTimeToSeg(end,&lastSeg,&segTime))
+        return false;
+
+    /* Which unique ref videos do we need to check for timebase compatibility? */
+    std::vector <uint32_t> ListOfRefs;
+
+    for(uint32_t i=firstSeg; i<=lastSeg; i++)
+    {
+        _SEGMENT *s=_segments.getSegment(i);
+        ADM_assert(s);
+        bool skip=false;
+        for(int i=0;i<ListOfRefs.size();i++)
+        {
+            if(ListOfRefs.at(i)==s->_reference)
+                skip=true;
+        }
+        if(skip) continue;
+        ListOfRefs.push_back(s->_reference);
+    }
+    if(ListOfRefs.empty())
+        return false;
+    uint32_t myscale,myrate;
+    for(int i=0;i<ListOfRefs.size();i++)
+    {
+        _VIDEOS *vid=_segments.getRefVideo(ListOfRefs.at(i));
+        ADM_assert(vid);
+        ADM_info("Ref video %u is %s-encoded, copy mode: %d\n",ListOfRefs.at(i),vid->fieldEncoded? "field" : "frame",copyMode);
+        vidHeader *demuxer=vid->_aviheader;
+        ADM_assert(demuxer);
+        aviInfo info;
+        demuxer->getVideoInfo(&info);
+        if(!i)
+        {
+            myscale=info.timebase_num;
+            myrate=info.timebase_den;
+            if(false==checkStdTimeBase(myscale,myrate))
+                return false;
+            if(vid->fieldEncoded && !copyMode)
+            {
+                switch(myrate)
+                {
+                    case 60000:
+                    case 50000:
+                    case 48000:
+                        ADM_info("Halving timebase denominator %u for field-encoded ref video.\n",myrate);
+                        myrate/=2;
+                        break;
+                    default:
+                        ADM_info("Doubling timebase numerator %u for field-encoded ref video.\n",myscale);
+                        myscale*=2;
+                        break;
+                }
+            }
+            continue;
+        }
+        /* FIXME properly reduce fractions when checking timebase */
+        if(false==checkStdTimeBase(info.timebase_num,info.timebase_den))
+            return false;
+        if(vid->fieldEncoded && !copyMode)
+        {
+            switch(info.timebase_den)
+            {
+                case 60000:
+                case 50000:
+                case 48000:
+                    ADM_info("Halving timebase denominator %u for appended field-encoded ref video.\n",info.timebase_den);
+                    info.timebase_den/=2;
+                    break;
+                default:
+                    ADM_info("Doubling timebase numerator %u for appended field-encoded video.\n",info.timebase_num);
+                    info.timebase_num*=2;
+                    break;
+            }
+        }
+        if(info.timebase_den == myrate)
+        {
+            if(info.timebase_num < myscale)
+            {
+                if(info.timebase_num > 1 && !(myscale % info.timebase_num))
+                    myscale = info.timebase_num;
+                else
+                    myscale = 1;
+            }
+        }else if(info.timebase_den > myrate)
+        {
+            if(!(info.timebase_den % myrate))
+            {
+                uint32_t mult = info.timebase_den / myrate;
+                myrate = info.timebase_den;
+                myscale *= mult;
+            }else
+            {
+                ADM_warning("Timebase mismatch: %u / %u (new) %u / %u (old)\n",info.timebase_num,info.timebase_den,myscale,myrate);
+                myrate = MAX_STD_CLOCK_FREQ;
+                myscale = 1;
+                break;
+            }
+        }else
+        {
+            if(myrate % info.timebase_den)
+            {
+                ADM_warning("Timebase mismatch: %u / %u (new) %u / %u (old)\n",info.timebase_num,info.timebase_den,myscale,myrate);
+                myrate = MAX_STD_CLOCK_FREQ;
+                myscale = 1;
+                break;
+            }
+        }
+    }
+    ADM_info("Timebase set to %u / %u\n",myscale,myrate);
+    *scale=myscale;
+    *rate=myrate;
+
+    return true;
+}
+
 /**
     \fn addFile
     \brief	Load or append a file.	The file type is determined automatically and the ad-hoc video decoder is spawned
@@ -335,7 +566,7 @@ uint8_t ADM_Composer::addFile (const char *name)
   else
     {
         // Read and construct the audio tracks for that videos
-      uint32_t extraLen;
+      uint32_t extraLen=0;
       uint8_t  *extraData;
       ADM_audioStream *stream;
       WAVHeader *header;
@@ -386,7 +617,14 @@ uint8_t ADM_Composer::addFile (const char *name)
             }
 
             if((track->wavheader).encoding==WAV_AAC)
+            {
+                uint32_t samples=AAC_DEFAULT_FRAME_LENGTH;
+                AacAudioInfo aac;
                 track->isbr=checkSamplingFrequency(track);
+                if(track->isbr || (ADM_getAacInfoFromConfig(extraLen,extraData,aac) && aac.sbr))
+                    samples<<=1;
+                stream->setSamplesPerPacket(samples);
+            }
 
             thisVid->audioTracks.push_back(track);
             if(!_segments.getNbRefVideos()) // 1st video..
@@ -567,10 +805,26 @@ uint32_t ADM_Composer::getPARHeight()
 /**
     \fn getFrameIncrement
 */
-uint64_t ADM_Composer::getFrameIncrement(void)
+uint64_t ADM_Composer::getFrameIncrement(bool copyMode)
 {
-    if (!_segments.getNbSegments()) return 0;
-    return _segments.getRefVideo(0)->timeIncrementInUs;
+    int nb = _segments.getNbRefVideos();
+    if(!nb) return 0;
+    uint64_t minIncrement = 0;
+    for(int i=0; i<nb; i++)
+    {
+        _VIDEOS *vid = _segments.getRefVideo(i);
+        ADM_assert(vid);
+        uint64_t inc = vid->timeIncrementInUs;
+        if(!inc) continue;
+        if(!copyMode && vid->fieldEncoded)
+        {
+            ADM_info("Doubling frame increment for field-encoded ref video %d\n",i);
+            inc *= 2;
+        }
+        if(!minIncrement | (inc < minIncrement))
+            minIncrement = inc;
+    }
+    return minIncrement;
 }
 /**
 	Set decoder settings (post process/swap u&v...)
