@@ -77,7 +77,6 @@ class dxvaRender: public VideoRenderBase,public ADM_QvideoDrawer
 
   protected:
                         admMutex        lock;
-                        bool            failure;
                         GUI_WindowInfo  info;
                         IDirect3DSurface9 *mySurface;
                         IDirect3DSurface9 *myYV12Surface;
@@ -116,7 +115,6 @@ dxvaRender::dxvaRender()
     videoBuffer=NULL;
     videoWidget=NULL;
     d3dHandle=admD3D::getHandle();
-    failure=false;
 }
 /**
     \fn dxvaRender
@@ -570,54 +568,49 @@ bool dxvaRender::displayImage_argb(ADMImage *pic)
 */
 bool dxvaRender::displayImage_surface(ADMImage *pic,admDx2Surface *surface)
 {
+#if 0
   // this does not work, both surfaces are coming from different device
-
-    if(!failure)
-    {
-        IDirect3DSurface9 *bBuffer;
-        if( ADM_FAILED(D3DCall(IDirect3DDevice9,GetBackBuffer,d3dDevice, 0, 0,
+    IDirect3DSurface9 *bBuffer;
+    if( ADM_FAILED(D3DCall(IDirect3DDevice9,GetBackBuffer,d3dDevice, 0, 0,
                                D3DBACKBUFFER_TYPE_MONO,
                                &bBuffer)))
-        {
-            ADM_warning("D3D Cannot create backBuffer\n");
-            return false;
-        }
-        // OK
-        ADM_info("surface duplicated\n");
+    {
+        ADM_warning("D3D Cannot create backBuffer\n");
+        return false;
+    }
+    // OK
+    ADM_info("surface duplicated\n");
 
-        // can we directly use the surface from dxva ? (can we at all ?)
-        if (ADM_FAILED(D3DCall(IDirect3DDevice9,StretchRect,d3dDevice,
+    // can we directly use the surface from dxva ? (can we at all ?)
+    if (ADM_FAILED(D3DCall(IDirect3DDevice9,StretchRect,d3dDevice,
                                surface->surface,
                                NULL,
                                bBuffer,
                                NULL,
                                D3DTEXF_LINEAR)))
-        {
-            ADM_warning("StretchRec yv12 failed\n");
-            failure=true;
-        }
+    {
+        ADM_warning("StretchRec yv12 failed\n");
     }else
     {
-        // go to indirect route
-        if(!pic->hwDownloadFromRef())
+        IDirect3DDevice9_BeginScene(d3dDevice);
+        IDirect3DDevice9_EndScene(d3dDevice);
+        if( ADM_FAILED(IDirect3DDevice9_Present(d3dDevice, &targetRect, 0, 0, 0)))
         {
-            ADM_warning("Failed to download yv12 from dxva\n");
-            return false;
+            ADM_warning("D3D Present failed\n");
         }
-        // workaround : use default non bridged path
-        if(useYV12)
-        {
-            return displayImage_yv12(pic);
-        }
-        return displayImage_argb(pic);
+        return true;
     }
-    IDirect3DDevice9_BeginScene(d3dDevice);
-    IDirect3DDevice9_EndScene(d3dDevice);
-    if( ADM_FAILED(IDirect3DDevice9_Present(d3dDevice, &targetRect, 0, 0, 0)))
+#endif
+    // go to indirect route
+    if(!pic->hwDownloadFromRef())
     {
-        ADM_warning("D3D Present failed\n");
+        ADM_warning("Failed to download yv12 from dxva\n");
+        return false;
     }
-    return true;
+    // workaround : use default non bridged path
+    if(useYV12)
+        return displayImage_yv12(pic);
+    return displayImage_argb(pic);
 }
 
 /**
