@@ -19,10 +19,10 @@
 
 
 #include "prefs.h"
-#include "DIA_working.h"
 #include "DIA_encoding.h"
 #include "DIA_coreToolkit.h"
 #include "ADM_vidMisc.h"
+#include "ADM_misc.h"
 #include "ADM_toolkitQt.h"
 #include "GUI_ui.h"
 #include "ADM_coreUtils.h"
@@ -84,6 +84,27 @@ void DIA_encodingQt4::shutdownChanged(int state)
 
 		GUI_Error_HIG(QT_TRANSLATE_NOOP("qencoding","Privileges Required"), QT_TRANSLATE_NOOP("qencoding","Root privileges are required to perform this operation."));
 	}
+#else
+	if(state)
+	{
+		ui->checkBoxKeepOpen->disconnect(SIGNAL(stateChanged(int)));
+		ui->checkBoxKeepOpen->setCheckState(Qt::Unchecked);
+		stayOpen=false;
+		connect(ui->checkBoxKeepOpen, SIGNAL(stateChanged(int)), this, SLOT(keepOpenChanged(int)));
+	}
+#endif
+}
+
+void DIA_encodingQt4::keepOpenChanged(int state)
+{
+    stayOpen=!!state;
+#ifdef _WIN32
+    if(state)
+    {
+        ui->checkBoxShutdown->disconnect(SIGNAL(stateChanged(int)));
+        ui->checkBoxShutdown->setCheckState(Qt::Unchecked);
+        connect(ui->checkBoxShutdown, SIGNAL(stateChanged(int)), this, SLOT(shutdownChanged(int)));
+    }
 #endif
 }
 
@@ -92,6 +113,7 @@ static char stringMe[80];
 DIA_encodingQt4::DIA_encodingQt4(uint64_t duration) : DIA_encodingBase(duration)
 {
         stopRequest=false;
+        stayOpen=false;
         UI_getTaskBarProgress()->enable();
         ui=new Ui_encodingDialog;
 	ui->setupUi(this);
@@ -108,10 +130,10 @@ DIA_encodingQt4::DIA_encodingQt4(uint64_t duration) : DIA_encodingBase(duration)
             ui->comboBoxPriority->setVisible(false);
             ui->labelPrio->setVisible(false);
         }
+        ui->checkBoxShutdown->setVisible(false);
 #endif
-        ui->checkBoxShutdown->setVisible(false); // shutdown is disabled, hide the checkbox on all platforms
-
 	connect(ui->checkBoxShutdown, SIGNAL(stateChanged(int)), this, SLOT(shutdownChanged(int)));
+	connect(ui->checkBoxKeepOpen, SIGNAL(stateChanged(int)), this, SLOT(keepOpenChanged(int)));
 	connect(ui->pushButton1, SIGNAL(pressed()), this, SLOT(useTrayButtonPressed()));
 	connect(ui->pushButton2, SIGNAL(pressed()), this, SLOT(pauseButtonPressed()));
 	connect(ui->comboBoxPriority, SIGNAL(currentIndexChanged(int)), this, SLOT(priorityChanged(int)));
@@ -154,21 +176,26 @@ DIA_encodingQt4::~DIA_encodingQt4( )
 {
     ADM_info("Destroying encoding qt4\n");
     UI_getTaskBarProgress()->disable();
-    //bool shutdownRequired = (ui->checkBoxShutdown->checkState() == Qt::Checked);
+    bool shutdownRequested = (ui->checkBoxShutdown->checkState() == Qt::Checked);
     if(tray)
     {
         UI_deiconify();
         delete tray;
         tray=NULL;
     }
-        
-	
-	if(ui) 
-        {
-            qtUnregisterDialog(this);
-            delete ui;
-            ui=NULL;
-        }
+    if(ui)
+    {
+        qtUnregisterDialog(this);
+        delete ui;
+        ui=NULL;
+    }
+    if(shutdownRequested)
+    {
+        prefs->save(); // won't get another chance
+        ADM_info("Requesting shutdown...\n");
+        if(!ADM_shutdown())
+            ADM_warning("Shutdown request failed\n");
+    }
 }
 /**
     \fn setPhasis(const char *n)
@@ -181,16 +208,12 @@ void DIA_encodingQt4::setPhasis(const char *n)
     if(!strcmp(n,"Pass 1"))
     {
         ui->tabWidget->setTabEnabled(1, false); // disable the "Advanced" tab
-        ui->checkBoxShutdown->setVisible(false); // hide the shutdown checkbox
         this->setWindowTitle(QString::fromUtf8(QT_TRANSLATE_NOOP("qencoding","First Pass")));
         WRITEM(labelPhasis,QT_TRANSLATE_NOOP("qencoding","Pass 1"));
     }else
     {
         this->setWindowTitle(QString::fromUtf8(QT_TRANSLATE_NOOP("qencoding","Encoding...")));
         ui->tabWidget->setTabEnabled(1, true);
-#if 0
-        ui->checkBoxShutdown->setVisible(true);
-#endif
         WRITEM(labelPhasis,n);
     }
 }
@@ -367,33 +390,67 @@ void DIA_encodingQt4::setRemainingTimeMS(uint32_t milliseconds)
 
 /**
     \fn isAlive( void )
-    \brief return 0 if the window was killed or cancel button press, 1 otherwisearchForward
+    \brief return 0 if the window was killed or cancel button pressed, 1 otherwise
 */
 bool DIA_encodingQt4::isAlive( void )
 {
+    if(!stopRequest)
+        return true;
 
-        if(stopRequest)
-        {
-                if(GUI_Alternate((char*)QT_TRANSLATE_NOOP("qencoding","The encoding is paused. Do you want to resume or abort?"),
-                              (char*)QT_TRANSLATE_NOOP("qencoding","Resume"),(char*)QT_TRANSLATE_NOOP("qencoding","Abort")))
-                 {
-                         stopRequest=false;
-                 }
-        }
-
-        if(!stopRequest) return true;		
-
-        return false;
+    if(GUI_Alternate((char*)QT_TRANSLATE_NOOP("qencoding","The encoding is paused. Do you want to resume or abort?"),
+                     (char*)QT_TRANSLATE_NOOP("qencoding","Resume"),(char*)QT_TRANSLATE_NOOP("qencoding","Abort")))
+    {
+        stopRequest=false;
+        return true;
+    }
+    ui->checkBoxShutdown->setCheckState(Qt::Unchecked);
+    return false;
 }
+
+/**
+    \fn keepOpen
+    \brief Allow user to read the muxing stats calmly
+*/
+void DIA_encodingQt4::keepOpen(void)
+{
+    if(stayOpen)
+    {
+        stopRequest=true;
+        UI_getTaskBarProgress()->disable();
+        if(tray)
+        {
+            UI_deiconify();
+            delete tray;
+            tray=NULL;
+        }
+        ui->checkBoxShutdown->setCheckState(Qt::Unchecked);
+        ui->checkBoxShutdown->setEnabled(false);
+        ui->pushButton1->setEnabled(false);
+        ui->pushButton2->setEnabled(false);
+        ui->comboBoxPriority->setEnabled(false);
+        show();
+        while(stayOpen)
+        {
+            ADM_usleep(100*1000);
+            QCoreApplication::processEvents();
+        }
+    }
+}
+
 /**
  * \fn closeEvent
  * @param event
  */
- void DIA_encodingQt4::closeEvent(QCloseEvent *event) 
+void DIA_encodingQt4::closeEvent(QCloseEvent *event)
+{
+    if(stopRequest && stayOpen)
     {
-        stopRequest=true;
-        event->ignore(); // keep window
+        stayOpen=false;
+        return;
     }
+    stopRequest=true;
+    event->ignore(); // keep window
+}
 
 /**
  * \fn reject
@@ -401,6 +458,11 @@ bool DIA_encodingQt4::isAlive( void )
  */
 void DIA_encodingQt4::reject(void)
 {
+    if(stopRequest && stayOpen) // close dialog when done
+    {
+        stayOpen=false;
+        return;
+    }
     stopRequest=true;
 }
 
