@@ -37,7 +37,7 @@ WAVHeader *asfHeader::getAudioInfo(uint32_t i )
   if(!_nbAudioTrack) return NULL;
   
   ADM_assert(i<_nbAudioTrack);
-  if(!_audioAccess) return NULL;
+  if(NULL==_audioAccess) return NULL;
   return &(_allAudioTracks[i].wavHeader);
 }
 /**
@@ -155,7 +155,7 @@ uint8_t asfHeader::open(const char *name)
   _fd=ADM_fopen(name,"rb");
   if(!_fd)
   {
-    GUI_Error_HIG("File Error.","Cannot open file\n");
+    GUI_Error_HIG(QT_TRANSLATE_NOOP("asfdemuxer","File Error."),QT_TRANSLATE_NOOP("asfdemuxer","Cannot open file\n"));
     return 0; 
   }
   myName=ADM_strdup(name);
@@ -167,7 +167,18 @@ uint8_t asfHeader::open(const char *name)
   for(int i=0;i<_nbAudioTrack;i++)
     ADM_info("Stream Audio: index=%d, sid=%d\n",
                 (int)_allAudioTracks[i].streamIndex,(int)_allAudioTracks[i].streamIndex);
-  buildIndex();
+  uint8_t r=buildIndex();
+  switch(r)
+  {
+      case 0:
+          ADM_error("No images found \n");
+          return r;
+      case ADM_IGN:
+          ADM_warning("Indexing cancelled by user after %" PRIu32" images\n",nbImage);
+          return r;
+      default:
+          break;
+  }
   fseeko(_fd,_dataStartOffset,SEEK_SET);
   _packet=new asfPacket(_fd,_nbPackets,_packetSize,&readQueue,&storageQueue,_dataStartOffset);
   curSeq=1;
@@ -176,11 +187,6 @@ uint8_t asfHeader::open(const char *name)
         _audioAccess[i]=new asfAudioAccess(this,i);
         _audioStreams[i]=ADM_audioCreateStream(&(_allAudioTracks[i].wavHeader), _audioAccess[i]);
   }
-  if(!nbImage)
-    {
-        ADM_error("No image found \n");
-        return 0;
-    }
   return 1;
 }
 /**
@@ -280,7 +286,7 @@ uint8_t  asfHeader::getFrame(uint32_t framenum,ADMCompressedImage *img)
   // Seeking ?
   if(_index[framenum].segNb!=curSeq || _index[framenum].segNb==1)
   {
-    printf("Seeking.. curseq:%u wanted seq:%u packet=%d\n",curSeq,_index[framenum].segNb,_index[framenum].packetNb);
+    aprintf("Seeking.. curseq:%u wanted seq:%u packet=%" PRIu64"\n",curSeq,_index[framenum].segNb,_index[framenum].packetNb);
     if(!_packet->goToPacket(_index[framenum].packetNb))
     {
       printf("[ASF] Cannot seek to frame %u\n",framenum);
@@ -320,7 +326,7 @@ uint8_t  asfHeader::getFrame(uint32_t framenum,ADMCompressedImage *img)
             bit=NULL;
             if(delta<230)
             {
-              printf("[ASF] Very suspicious delta :%"PRIu32"\n",delta);
+              printf("[ASF] Very suspicious delta :%" PRIu32"\n",delta);
             }
             continue; 
           }
@@ -360,6 +366,7 @@ gotcha:
   img->dataLength=len;
   img->demuxerDts=_index[framenum].dts;
   img->demuxerPts=_index[framenum].pts;
+  img->flags=_index[framenum].flags;
   if(len!=_index[framenum].frameLen)
   {
     ADM_error("[ASF] Frame=%u :-> Mismatch found len : %u expected %u\n",framenum,len, _index[framenum].frameLen);
