@@ -36,7 +36,7 @@ static const frameRateStruct candidateFrameRate[]=
    {40000,1000,25000}, // 25 fps
    {33367,1001,30000}, // 30 NTSC
    //{20853,2000,24000}, // 24*2
-   {20854,2002,24000}, // 23976*2
+   {20854,1001,48000}, // 23976*2
    {20000,1000,50000}, // 50
    {16683,1001,60000}, // 60 NTSCs
    {16667,1000,60000} // 60 fps
@@ -114,11 +114,23 @@ uint8_t mkvHeader::open(const char *name)
   _segmentPosition=ebml.tell();
   ADM_info("[MKV] found Segment at 0x%llx\n",(uint64_t)_segmentPosition);
    /* Now find tracks */
+  bool tracksElemFound=false;
+  uint64_t nextHead=0;
   if(ebml.find(ADM_MKV_SECONDARY,MKV_SEGMENT,MKV_SEEK_HEAD,&alen))
   {
        ADM_ebml_file seekHead( &ebml,alen);
-       readSeekHead(&seekHead);
-  }else      
+       tracksElemFound = readSeekHead(&seekHead,&nextHead);
+       if(!tracksElemFound && nextHead)
+       {
+           ebml.seek(nextHead);
+           if(ebml.simplefind(MKV_SEEK_HEAD,&len,false))
+           {
+               ADM_ebml_file secondary(&ebml,len);
+               tracksElemFound = readSeekHead(&secondary);
+           }
+       }
+  }
+  if(!tracksElemFound)
   { // No valid seek_head, try to find MKV_TRACKS
       ADM_info("Searching for MKV_TRACKS\n");
       int headerLen;
@@ -183,6 +195,7 @@ uint8_t mkvHeader::open(const char *name)
         ADM_info("Track %" PRIu32" has an index size of %d entries\n",i,_tracks[i].index.size());
 
     int last=_tracks[0].index.size();
+#if 0
     if(isVC1Compatible(_videostream.fccHandler))
     {
         mkvTrak *vid=_tracks;
@@ -191,6 +204,7 @@ uint8_t mkvHeader::open(const char *name)
         for(int i=1;i<last-1;i++)
             vid->index[i].Pts=ADM_NO_PTS;
     }
+#endif
   // Delay frames + recompute frame duration
 // now that we have a good frameduration and max pts dts difference, we can set a proper DTS for all video frame
     uint32_t ptsdtsdelta, mindelta;
@@ -204,56 +218,9 @@ uint8_t mkvHeader::open(const char *name)
     mkvTrak *vid=_tracks;
     if(hasBframe==true) // Try to compute a sane DTS knowing the PTS and the DTS/PTS delay
     {
-        bool useSortedPtsToSetDts=false;
-        int bumped=0;
-        for(int i=1;i<last;i++)
-        {
-            uint64_t pts;
-            pts=vid->index[i].Pts;
-            lastDts+=increment; // This frame dts with no correction
-            if(pts==ADM_NO_PTS)
-            {
-                vid->index[i].Dts=lastDts;
-                continue;
-            }
-            uint64_t limitDts=0;
-            if(vid->index[i].Pts>ptsdtsdelta)
-                limitDts=vid->index[i].Pts-ptsdtsdelta;
+        setDtsFromListOfSortedPts();
 
-#define DTS_BUMPED_THRESHOLD 10
-
-            if(lastDts<limitDts)
-            {
-                if(i==1)
-                {
-                    lastDts = (vid->index[0].Pts-ptsdtsdelta>0)? vid->index[0].Pts-ptsdtsdelta : 0;
-                    vid->index[0].Dts = lastDts;
-                    lastDts+=increment;
-                }
-                if(limitDts-lastDts>1000) // not just a rounding error
-                {
-                    ADM_warning("Bumping DTS by %" PRIu64" ms to keep PTS/DTS delta within limits for frame %d\n",(limitDts-lastDts)/1000,i);
-                    lastDts=limitDts;
-                    bumped++;
-                    if(bumped>DTS_BUMPED_THRESHOLD)
-                    {
-                        ADM_warning("Default time increment is probably too low, resorting to direct mapping.\n");
-                        useSortedPtsToSetDts=true;
-                        break;
-                    }
-                }
-            }
-            vid->index[i].Dts=lastDts;
-        }
-        if(useSortedPtsToSetDts)
-        {
-            setDtsFromListOfSortedPts();
-        }
-        // Check that we have PTS>=DTS also
-        uint64_t enforcePtsGreaterThanDts;
-        bool secondTry=false;
-_again:
-        enforcePtsGreaterThanDts=0;
+        uint64_t enforcePtsGreaterThanDts=0;
         int frmeMaxDlta=-1;
         for(int i=0;i<last;i++)
         {
@@ -270,17 +237,6 @@ _again:
         if(enforcePtsGreaterThanDts)
         {
             ADM_info("Have to delay by %" PRIu64" us detected for frame %d so that PTS>DTS\n",enforcePtsGreaterThanDts,frmeMaxDlta);
-            if(enforcePtsGreaterThanDts>ptsdtsdelta)
-            {
-                ADM_warning("The delay is implausibly high. The calculated frame rate is probably too low.\n");
-                if(!secondTry)
-                {
-                    setDtsFromListOfSortedPts();
-                    ADM_warning("Retrying with direct mapping.\n");
-                    secondTry=true;
-                    goto _again;
-                }
-            }
             for(int i=0;i<_nbAudioTrack+1;i++)
                 delayTrack(i,&(_tracks[i]),enforcePtsGreaterThanDts);
         }
@@ -310,6 +266,8 @@ _again:
           for(int i=0;i<_nbAudioTrack;i++)
           {
             rescaleTrack(&(_tracks[1+i]),duration32);
+            if(!_tracks[1+i].duration)
+                _tracks[1+i].duration=_tracks[0].duration; // FIXME
             switch(_tracks[1+i].wavHeader.encoding)
             {
                 case WAV_OGG_VORBIS:            
@@ -337,10 +295,14 @@ _again:
         {
             mkvAccess *access=new mkvAccess(_filename,&(_tracks[i+1]));
             _access[i]=new mkvAccessBuffered(access,_tracks[i+1]._needBuffering);
-        }
-        else
+        }else if(_tracks[i+1].wavHeader.encoding==MKV_MUX_LATM)
         {
-               _access[i]=new mkvAccess(_filename,&(_tracks[i+1]));
+            mkvAccess *access=new mkvAccess(_filename,&(_tracks[i+1]));
+            _access[i]=new mkvAccessLatm(access,LATM_MAX_BUFFER_SIZE);
+            _tracks[i+1].wavHeader.encoding=WAV_AAC;
+        }else
+        {
+            _access[i]=new mkvAccess(_filename,&(_tracks[i+1]));
         }
         _audioStreams[i]=ADM_audioCreateStream(&(_tracks[1+i].wavHeader), _access[i]);;
         _audioStreams[i]->setLanguage(_tracks[1+i].language);
@@ -452,11 +414,19 @@ bool mkvHeader::enforceFixedFrameRate(int num, int den)
 {
   mkvTrak *track=_tracks;
   int nb=track->index.size();
+  ADM_assert(den);
   double dHalf=(500000.*(double)num)/((double)den);
   int half=dHalf-1; // half interval in us
   int first=0;
   while(  track->index[first].Pts==ADM_NO_PTS && first<nb) first++; // we should have some at least
   uint64_t zero= track->index[first].Pts;
+  {
+    double dmultiple=zero+half;
+    dmultiple*=den;
+    dmultiple/=(1000000.*(double)num);
+    zero=((uint64_t)dmultiple*1000000*num)/den;
+    track->index[first].Pts=zero;
+  }
   ADM_info("Num=%d Den=%d half=%d zero=%d first=%d\n",num,den,half,(int)zero,first);
   for(int i=first+1;i<nb;i++)
   {
@@ -477,6 +447,13 @@ bool mkvHeader::enforceFixedFrameRate(int num, int den)
 #endif
     track->index[i].Pts=reconstructed;
   }
+  _videostream.dwScale=num;
+  _videostream.dwRate=den;
+  double f=num;
+  f=f*1000000.;
+  f/=den;
+  f+=0.49;
+  track->_defaultFrameDuration=f;
   return true;
 }
 /**
@@ -545,7 +522,34 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
     if(nbBFrame) *bFramePresent=true;
 
     ADM_warning(">>> MinDelta=%d MaxDelta=%d\n",minDelta,maxDelta);
-    int stdFrameRate=getStdFrameRate(track->_defaultFrameDuration);
+    // Check whether the default frame duration is valid
+    int stdFrameRate=-1;
+    if(!nbBFrame && minDelta)
+    {
+        bool frameDurationInvalid=false;
+        if((uint64_t)maxDelta < (uint64_t)track->_defaultFrameDuration)
+        {
+            ADM_warning("Default frame duration is invalid, exceeds max delta\n");
+            frameDurationInvalid=true;
+        }
+        if((uint64_t)minDelta > (uint64_t)track->_defaultFrameDuration)
+        {
+            ADM_warning("Default frame duration is invalid, below min delta\n");
+            frameDurationInvalid=true;
+        }
+        if(frameDurationInvalid)
+        {
+            stdFrameRate=getStdFrameRate(minDelta);
+            if(stdFrameRate!=-1 && stdFrameRate==getStdFrameRate(maxDelta)) // full confidence
+            {
+                _videostream.dwScale = candidateFrameRate[stdFrameRate].num;
+                _videostream.dwRate = candidateFrameRate[stdFrameRate].den;
+                track->_defaultFrameDuration = candidateFrameRate[stdFrameRate].durationInUs;
+            }
+        }
+    }
+
+    stdFrameRate=getStdFrameRate(track->_defaultFrameDuration);
 
     int num= _videostream.dwScale;
     int den= _videostream.dwRate;
@@ -633,6 +637,8 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
                 deviation=deviationMinDelta;
                 skipped=minDeltaSkip;
                 ADM_info("Min delta is better\n");
+                if(stdFrameRate==-1)
+                    stdFrameRate=getStdFrameRate(minDelta);
             }
         }
 
@@ -644,20 +650,31 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
             ADM_info("Checking deviation for stdFrameRate=%d:%d\n",fr->num,fr->den);
             int deviationStd=devEngine.computeDeviation(fr->num,fr->den,stdSkip);
             ADM_info("Deviation for stdFrameRate(%d) =%d\n",stdFrameRate,deviationStd);
-            if(deviationStd<deviation && stdSkip<skipped*3)
+            if(deviationStd<=deviation && stdSkip<=skipped*3)
             {
               num=fr->num;
               den=fr->den;
               deviation=deviationStd;
-              ADM_info("Std frame rate is better\n");
+              skipped=stdSkip;
+              ADM_info("Std frame rate is equal or better\n");
             }
         }
     }
     ADM_info("Old default duration    %" PRId64" us\n",track->_defaultFrameDuration);
-    if(!deviation)
+#define TOLERANCE 500
+    if(deviation < TOLERANCE)
     {
         ADM_info("We are within margin, recomputing timestamp with exact value (%d vs %d)\n",num,den);
         enforceFixedFrameRate(num,den);
+        if(!skipped)
+        {
+            ADM_info("Enforcing constant frame rate.\n");
+            _mainaviheader.dwMicroSecPerFrame=0; // perfectly regular
+        }else
+        {
+            ADM_info("Variable frame rate with time base %d/%d\n",num,den);
+            _mainaviheader.dwMicroSecPerFrame=track->_defaultFrameDuration;
+        }
         // do it again, the old may not be valid 
         // Search minimum and maximum between 2 frames
         // the minimum will give us the maximum fps
@@ -681,25 +698,20 @@ bool mkvHeader::ComputeDeltaAndCheckBFrames(uint32_t *minDeltaX, uint32_t *maxDe
             //printf("\/=%" PRId64" Min %" PRId64" MAX %" PRId64"\n",delta,minDelta,maxDelta);
         }
 
-    }
-
-    //if( num!= _videostream.dwScale ||  den!= _videostream.dwRate)
+    }else
     {
-        ADM_info("New framerate values : %d:%d\n",num,den);
-        _videostream.dwScale=num;
-        _videostream.dwRate=den;
+        ADM_info("Variable frame rate, using 1/90000 time base.\n");
+        _videostream.dwScale=1;
+        _videostream.dwRate=90000;
         double f=num;
         f=f*1000000.;
         f/=den;
-        track->_defaultFrameDuration=f;
-       
-
+        f+=0.49;
+        track->_defaultFrameDuration=_mainaviheader.dwMicroSecPerFrame=(uint32_t)f;
     }
 
     ADM_info("New default duration    %" PRId64" us\n",track->_defaultFrameDuration);
-
-
-
+#undef TOLERANCE
     ADM_info("First frame pts     %" PRId64" us\n",track->index[0].Pts);
 
     *maxDeltaX=maxDelta;
@@ -936,7 +948,9 @@ uint8_t mkvHeader::close(void)
     _parser=NULL;
 
 
-#define FREEIF(i) { if(_tracks[i].extraData) delete [] _tracks[i].extraData; _tracks[i].extraData=0;}
+#define FREEIF(i) { if(_tracks[i].extraData) delete [] _tracks[i].extraData; _tracks[i].extraData=0; \
+                    if(_tracks[i].infoCache) delete [] _tracks[i].infoCache; _tracks[i].infoCache=0; \
+                    if(_tracks[i].paramCache) delete [] _tracks[i].paramCache; _tracks[i].paramCache=0; }
   if(_isvideopresent)
   {
       FREEIF(0);
@@ -958,6 +972,8 @@ uint8_t mkvHeader::close(void)
         delete [] _access;
         _access=NULL;
     }
+    ADM_dealloc(_filename);
+    _filename=NULL;
     return 1;
 }
 /**
@@ -1011,7 +1027,7 @@ uint32_t mkvHeader::getFlags(uint32_t frame,uint32_t *flags)
 {
   if(frame>=_tracks[0].index.size()) return 0;
   *flags=_tracks[0].index[frame].flags;
-  if(!frame) *flags=AVI_KEY_FRAME;
+  if(!frame) *flags=AVI_KEY_FRAME | (*flags & AVI_STRUCTURE_TYPE_MASK);
   return 1;
 }
 /**
@@ -1191,10 +1207,12 @@ bool    mkvHeader::setPtsDts(uint32_t frame,uint64_t pts,uint64_t dts)
  * \fn readSeekHead
  * \bried used to locate the interesting parts of the file
  */
-bool    mkvHeader::readSeekHead(ADM_ebml_file *body)
+bool    mkvHeader::readSeekHead(ADM_ebml_file *body, uint64_t *nexthead)
 {
     uint64_t vlen,len;
     ADM_info("Parsing SeekHead\n");
+    if(nexthead)
+        *nexthead=0;
     while(!body->finished())
     {
         if(!body->simplefind(MKV_SEEK,&vlen,false))
@@ -1241,6 +1259,14 @@ bool    mkvHeader::readSeekHead(ADM_ebml_file *body)
         uint64_t position=item.readUnsignedInt(len);
         switch(t)
         {
+            case MKV_SEEK_HEAD:
+                {
+                    uint64_t chained=position+_segmentPosition;
+                    ADM_info("Chained MKV_SEEK_HEAD at position %" PRIu64"\n",chained);
+                    if(nexthead)
+                        *nexthead=chained;
+                    break;
+                }
             case MKV_CUES:
                     _cuePosition=position+_segmentPosition;
                     ADM_info("   at position  0x%llx\n",_cuePosition);
