@@ -24,6 +24,7 @@
 
 #include "ADM_Video.h"
 #include "ADM_audioStream.h"
+#include "ADM_aacLatm.h"
 #include "ADM_ebml.h"
 #include <BVector.h>
 #define MKV_MAX_REPEAT_HEADER_SIZE 16
@@ -58,6 +59,10 @@ public:
         length=0;
         extraData=NULL;
         extraDataLen=0;
+        infoCache=NULL;
+        infoCacheSize=0;
+        paramCache=NULL;
+        paramCacheSize=0;
         headerRepeatSize=0;
         _sizeInBytes=0;
         _defaultFrameDuration=0;
@@ -76,6 +81,10 @@ public:
   /* Used for both */
   uint8_t    *extraData;
   uint32_t   extraDataLen;
+  uint8_t    *infoCache; // decoded SPS & friends
+  uint32_t   infoCacheSize;
+  uint8_t    *paramCache; // raw SPS data
+  uint32_t   paramCacheSize;
   /* */
   uint32_t   headerRepeatSize;
   uint8_t    headerRepeat[MKV_MAX_REPEAT_HEADER_SIZE];
@@ -88,6 +97,7 @@ public:
 };
 
 #define MKV_MAX_LACES 101 // ?
+#define MKV_MUX_LATM 0x1602
 /**
  */
 class mkvAccess;
@@ -112,6 +122,26 @@ protected:
             int         _offset;
 };
 
+class mkvAccessLatm : public ADM_audioAccess
+{
+public:
+            mkvAccessLatm(mkvAccess *ac,int maxSize);
+            ~mkvAccessLatm();
+                virtual bool      isCBR(void) { return false; }
+                virtual bool      canSeekTime(void) { return true; };
+                virtual bool      canSeekOffset(void) { return false; };
+                virtual bool      canGetDuration(void) { return true; };
+                virtual uint64_t  getDurationInUs(void);
+                virtual bool      goToTime(uint64_t timeUs);
+                virtual bool      getPacket(uint8_t *buffer, uint32_t *size, uint32_t maxSize,uint64_t *dts);
+                virtual bool      getExtraData(uint32_t *l, uint8_t **d);
+protected:
+            ADM_latm2aac latm;
+            mkvAccess   *_son;
+            int         _maxSize;
+            uint8_t     *_buffer;
+};
+
 /**
     \class mkvAccess
     \brief Matroska audio demuxer
@@ -202,7 +232,7 @@ class mkvHeader         :public vidHeader
     uint8_t                 walk(void *seed);
     uint64_t                walkAndFind(void *seed,MKV_ELEM_ID searched);
     int                     searchTrackFromTid(uint32_t tid);
-    bool                    readSeekHead(ADM_ebml_file *body);
+    bool                    readSeekHead(ADM_ebml_file *body,uint64_t *nexthead=NULL);
     //
     //                      
     int                     isBufferingNeeded(mkvTrak *trk); // Split audio blocks into smaller pieces if needed
