@@ -88,7 +88,7 @@ bool TS_scanForPrograms(const char *file,uint32_t *nbTracks, ADM_TS_TRACK **outT
     nb=counter=failed=start=match=0;
 
     tsPacket *t=new tsPacket();
-    t->open(file,FP_PROBE);
+    t->open(file,1);
     int videoIndex=-1;
     int vidPidToMatch=-1;
     ADM_TS_TRACK_TYPE vidTypeToMatch=ADM_TS_UNKNOWN;
@@ -366,9 +366,10 @@ ADM_TS_TRACK_TYPE EsType(uint32_t type,const char **str)
                 case 0x1B:     *str= "H264 Video";return ADM_TS_H264;break;
                 case 0x24:     *str= "H265 Video(not sure)";return ADM_TS_H265;break;
                 case 0x81:     *str= "AC3 (Not sure)";return ADM_TS_AC3;break;
-                case 0x82:     *str= "DTS (Not sure)";return ADM_TS_DTS;break;
+                case 0x82:     *str= "DTS (BluRay)";return ADM_TS_DTS;break;
                 case 0x83:     *str= "TrueHD AC3  (BluRay)";return ADM_TS_AC3;break;
                 case 0x84:     *str= "E-AC3 (Not sure)";return ADM_TS_EAC3;break;
+                case 0x86:     *str= "DTS XLL (BluRay)";return ADM_TS_DTS;break;
                 case 0x87:     *str= "E-AC3 (Not sure)";return ADM_TS_EAC3;break;
                 case 0xea:     *str= "VC1 (Not sure)";return ADM_TS_VC1;break;
                 case 0x90:     *str= "Presentation graphics (BluRay)";return ADM_TS_UNKNOWN;break;
