@@ -23,7 +23,7 @@
 #include "DIA_coreToolkit.h"
 #include "ADM_odml_audio.h"
 #include "ADM_coreUtils.h"
-
+#include "ADM_codecType.h"
 #ifdef ADM_DEBUG
 	#define OPENDML_VERBOSE
     #define aprintf printf
@@ -294,7 +294,7 @@ uint32_t rd;
 		// now read up each parts...
 		//____________________________
 		                
-#define DUMP_TRACK(i) aprintf(" at %"PRIu64" (%"PRIx64") size : %"PRIu64" (%"PRIx64")\n", \
+#define DUMP_TRACK(i) aprintf(" at %" PRIu64" (%" PRIx64") size : %" PRIu64" (%" PRIx64")\n", \
 				_Tracks[i].strh.offset,\
 				_Tracks[i].strh.offset,\
 				_Tracks[i].strh.size,\
@@ -314,11 +314,11 @@ uint32_t rd;
 			fseeko(_fd,_Tracks[i].strh.offset,SEEK_SET);
 			if(_Tracks[i].strh.size!=sizeof(_videostream))
 			{
-				printf("[AVI]Mmm(1) we have a bogey here, size mismatch : %"PRIu64"\n",_Tracks[i].strh.size);
+				printf("[AVI]Mmm(1) we have a bogey here, size mismatch : %" PRIu64"\n",_Tracks[i].strh.size);
 				printf("[AVI]expected %d\n",(int)sizeof(_videostream));
 				if(_Tracks[i].strh.size<sizeof(_videostream)-8) // RECT is not mandatory
 				{
-                                  GUI_Error_HIG(QT_TR_NOOP("Malformed header"), NULL);
+                                  GUI_Error_HIG(QT_TRANSLATE_NOOP("opendmldemuxer","Malformed header"), NULL);
 					return 0;
 				}		
 				printf("[AVI]Trying to continue anyway\n");			
@@ -348,7 +348,7 @@ uint32_t rd;
 		extra=_Tracks[vidTrack].strf.size-sizeof(_video_bih);
 		if(extra<0)
 		{	
-			printf("[AVI]bih is not big enough (%"PRIu64"/%d)!\n",_Tracks[vidTrack].strf.size,(int)sizeof(_video_bih));
+			printf("[AVI]bih is not big enough (%" PRIu64"/%d)!\n",_Tracks[vidTrack].strf.size,(int)sizeof(_video_bih));
 			return 0;
 		}
 		fread(&_video_bih,sizeof(_video_bih),1,_fd);
@@ -390,11 +390,11 @@ uint32_t rd;
                                         fseeko(_fd,_Tracks[run].strh.offset,SEEK_SET);
                                         if(_Tracks[run].strh.size != sizeof(_audiostream))
                                         {
-                                                printf("[AVI]Mmm(2) we have a bogey here, size mismatch : %"PRIu64"\n",_Tracks[run].strh.size);
+                                                printf("[AVI]Mmm(2) we have a bogey here, size mismatch : %" PRIu64"\n",_Tracks[run].strh.size);
                                                 printf("[AVI]expected %d\n",(int)sizeof(_audiostream));
                                                 if(_Tracks[run].strh.size<sizeof(_audiostream)-8)
                                                 {
-                                                  GUI_Error_HIG(QT_TR_NOOP("Malformed header"), NULL);
+                                                  GUI_Error_HIG(QT_TRANSLATE_NOOP("opendmldemuxer","Malformed header"), NULL);
                                                         return 0;
                                                 }
                                                 printf("[AVI]Trying to continue anyway\n");			
@@ -414,7 +414,7 @@ uint32_t rd;
                                         extra=_Tracks[run].strf.size-sizeof(WAVHeader);
                                         if(extra<0)
                                         {	
-                                                printf("[AVI]WavHeader is not big enough (%"PRIu64"/%d)!\n",
+                                                printf("[AVI]WavHeader is not big enough (%" PRIu64"/%d)!\n",
                                                 _Tracks[run].strf.size,(int)sizeof(WAVHeader));
                                                 return 0;
                                         }
@@ -544,9 +544,14 @@ bool OpenDMLHeader::removeEmptyFrames(void)
     
     odmlIndex *nwIdx=new odmlIndex[_videostream.dwLength];
     int index=0;
+    bool isFraps=fourCC::check(_videostream.fccHandler,(uint8_t *)"FPS1");
+    
     for(int i=0;i<_videostream.dwLength;i++)
     {
-        if(_idx[i].size)
+        bool skip=false;
+        if(!_idx[i].size) skip=true;
+        if(isFraps && _idx[i].size==8) skip=true;
+        if(!skip)
         {
             nwIdx[index++]=_idx[i];
         }
@@ -559,7 +564,7 @@ bool OpenDMLHeader::removeEmptyFrames(void)
     }
     int delta=_videostream.dwLength-index;
 
-    printf("[openDml] Removed %d empty frames\n",(int)delta);
+    printf("[openDml] Removed %d empty frames, new total is %d\n",(int)delta,(int)index);
     _mainaviheader.dwTotalFrames=_videostream.dwLength=index;
     delete [] _idx;
     _idx=nwIdx;
@@ -578,7 +583,6 @@ bool OpenDMLHeader::removeEmptyFrames(void)
     \fn computePtsDts
     \brief Compute PtsDts
 */
-extern uint8_t isMpeg4Compatible (uint32_t fourcc);
 uint8_t OpenDMLHeader::computePtsDts(void)
 {
     // if it is mpeg4-sp, removed packet bitstream & reindex
@@ -654,11 +658,11 @@ uint32_t count=0;
 			if(_Tracks[i].strh.size!=sizeof(tmp))
 			{
 				
-				printf("[AVI]Mmm(3) we have a bogey here, size mismatch : %"PRIu64"\n",_Tracks[i].strh.size);
+				printf("[AVI]Mmm(3) we have a bogey here, size mismatch : %" PRIu64"\n",_Tracks[i].strh.size);
 				printf("[AVI]expected %d\n",(int)sizeof(tmp));
 				if(_Tracks[i].strh.size<sizeof(tmp)-8)
 				{
-                                  GUI_Error_HIG(QT_TR_NOOP("Malformed header"), NULL);
+                                  GUI_Error_HIG(QT_TRANSLATE_NOOP("opendmldemuxer","Malformed header"), NULL);
 					return 0;
 				}		
 				printf("[AVI]Trying to continue anyway\n");			
@@ -683,8 +687,8 @@ uint32_t count=0;
                                 }
                                 else
                                 {
-                                        printf("Track %u/%u :\n",i,_nbTrack);
-                                        fourCC::print(tmp.fccType);
+                                        printf("Track %u/%u : ",i,_nbTrack);
+                                        fourCC::print(tmp.fccType);printf(", ");
                                         fourCC::print(tmp.fccHandler);
                                         printf("\n");
                                 }
@@ -723,11 +727,10 @@ void OpenDMLHeader::Dump( void )
 				
 	printf(  "[Avi] video stream attached:\n" );
 	printf(  "[Avi] ______________________\n" );	
-	printf(  "[Avi] Extra Data  : %u",_videoExtraLen);
+	printf(  "[Avi] Extra Data  : %u\n",_videoExtraLen);
 	if(_videoExtraLen)
 	{
 		mixDump( _videoExtraData, _videoExtraLen);
-        printf("\n");
 	}
 
 	printf("[Avi]  fccType     :");
@@ -800,7 +803,6 @@ void OpenDMLHeader::Dump( void )
 	{
 		mixDump( _audioTracks[i].extraData, _audioTracks[i].extraDataLen);
 	}
-	printf("\n");
 
       }
 }
@@ -831,7 +833,7 @@ void OpenDMLHeader::walk(riffParser *p)
 				aprintf("main header found \n");
 				if(len!=sizeof(_mainaviheader))
 				{
-					printf("[AVI]oops : %"PRIu32" / %d\n",len,(int)sizeof(_mainaviheader));
+					printf("[AVI]oops : %" PRIu32" / %d\n",len,(int)sizeof(_mainaviheader));
 				}
 				p->read(len,(uint8_t *)&_mainaviheader);
 
@@ -842,7 +844,7 @@ void OpenDMLHeader::walk(riffParser *p)
 				break;
 		case MKFCC('i','d','x','1'):
                                 _regularIndex.offset=p->getPos();
-                                printf("[Avi] Idx1 found at offset %"PRIx64"\n",_regularIndex.offset);
+                                printf("[Avi] Idx1 found at offset %" PRIx64"\n",_regularIndex.offset);
                                 _regularIndex.size=len;
 				return;
 				break;				
@@ -946,7 +948,7 @@ bool    OpenDMLHeader::getPtsDts(uint32_t frame,uint64_t *pts,uint64_t *dts)
 {
     if(frame>_videostream.dwLength)
     {
-        printf("[Odml] %"PRIu32" exceeds nb of video frames %"PRIu32"\n",frame,_videostream.dwLength);
+        printf("[Odml] %" PRIu32" exceeds nb of video frames %" PRIu32"\n",frame,_videostream.dwLength);
         return false;
     }
     *dts=_idx[frame].dts; // FIXME
@@ -960,7 +962,7 @@ bool    OpenDMLHeader::setPtsDts(uint32_t frame,uint64_t pts,uint64_t dts)
 {
     if(frame>_videostream.dwLength)
     {
-        printf("[Odml] %"PRIu32" exceeds nb of video frames %"PRIu32"\n",frame,_videostream.dwLength);
+        printf("[Odml] %" PRIu32" exceeds nb of video frames %" PRIu32"\n",frame,_videostream.dwLength);
         return false;
     }
     _idx[frame].dts=dts;
