@@ -35,7 +35,8 @@ mp4_muxer muxerConfig=
     MP4_MUXER_OPT_FASTSTART,
     false,
     WIDE,
-    MP4_MUXER_ROTATE_0
+    MP4_MUXER_ROTATE_0,
+    MP4_MUXER_CLOCK_FREQ_AUTO
 };
 
 
@@ -55,6 +56,25 @@ muxerMP4::~muxerMP4()
 {
 
 }
+
+static uint32_t getClockFreqFromEnum(MP4_MUXER_CLOCK_FREQUENCIES type)
+{
+    switch(type)
+    {
+        case MP4_MUXER_CLOCK_FREQ_AUTO: return 0;
+        case MP4_MUXER_CLOCK_FREQ_24KHZ: return 24000;
+        case MP4_MUXER_CLOCK_FREQ_25KHZ: return 25000;
+        case MP4_MUXER_CLOCK_FREQ_30KHZ: return 30000;
+        case MP4_MUXER_CLOCK_FREQ_50KHZ: return 50000;
+        case MP4_MUXER_CLOCK_FREQ_60KHZ: return 60000;
+        case MP4_MUXER_CLOCK_FREQ_90KHZ: return 90000;
+        case MP4_MUXER_CLOCK_FREQ_180KHZ: return 180000;
+        default:
+            ADM_warning("Illegal type = %u\n",type);
+            return 0;
+    }
+}
+
 /**
     \fn open
     \brief Check that the streams are ok, initialize context...
@@ -62,24 +82,50 @@ muxerMP4::~muxerMP4()
 
 bool muxerMP4::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack,ADM_audioStream **a)
 {
-    if(!isMpeg4Compatible(s->getFCC()) && !isH264Compatible(s->getFCC()) && !isH265Compatible(s->getFCC()))
+    uint32_t fcc=s->getFCC();
+    if(!isMpeg4Compatible(fcc) && !isH264Compatible(fcc) && !isH265Compatible(fcc) && !fourCC::check(fcc,(const uint8_t *)"av01"))
     {
-            GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4muxer","Unsupported"),QT_TRANSLATE_NOOP("mp4muxer","Only MP4Video, H264, and H265 supported for video"));
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4muxer","Unsupported"),QT_TRANSLATE_NOOP("mp4muxer","Only MP4Video, H264, H265 and AV1 supported for video"));
             return false;
     }
+    bool muxModeMov=false;
     if(nbAudioTrack)
+    {
         for(int i=0;i<nbAudioTrack;i++)
         {
             uint32_t acc=a[i]->getInfo()->encoding;
-            if(acc!=WAV_MP2 && acc!=WAV_MP3 && acc!=WAV_AAC && acc!=WAV_AC3 && acc!=WAV_EAC3)
+            if( acc!=WAV_AAC &&
+                acc!=WAV_AC3 &&
+                acc!=WAV_EAC3 &&
+                acc!=WAV_LPCM &&
+                acc!=WAV_MP2 &&
+                acc!=WAV_MP3 &&
+                acc!=WAV_OGG_VORBIS)
             {
-                GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4muxer","Unsupported"),QT_TRANSLATE_NOOP("mp4muxer","Only AAC, AC3, E-AC3 and mpegaudio supported for audio"));
+                GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4muxer","Unsupported"),QT_TRANSLATE_NOOP("mp4muxer","Only AAC, AC3, E-AC3, LPCM, MP2, MP3 and Vorbis supported for audio"));
                 return false;
             }
+            if(acc==WAV_LPCM)
+                muxModeMov=true;
         }
+    }
     /* All seems fine, open stuff */
-    const char *f="mp4";
-    if(muxerConfig.muxerType==MP4_MUXER_PSP) f="psp";
+    const char *f;
+    if(muxerConfig.muxerType==MP4_MUXER_PSP)
+    {
+        if(muxModeMov)
+        {
+            GUI_Error_HIG(QT_TRANSLATE_NOOP("mp4muxer","Incompatible Format"),QT_TRANSLATE_NOOP("mp4muxer","PSP format is incompatible with LPCM audio"));
+            return false;
+        }
+        f="psp";
+    }else if(muxModeMov)
+    {
+        f="mov";
+    }else
+    {
+        f="mp4";
+    }
     if(false==setupMuxer(f,file))
     {
         printf("[MP4] Failed to open muxer\n");
@@ -98,9 +144,31 @@ bool muxerMP4::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack,A
         c = video_st->codec;
         AVCodecParameters *par;
         par = video_st->codecpar;
-        if(isH265Compatible(s->getFCC()))
+        if(isH265Compatible(fcc))
             par->codec_tag = MKTAG('h', 'v', 'c', '1');
-        rescaleFps(s->getAvgFps1000(),&(c->time_base));
+        uint32_t timescale=getClockFreqFromEnum((MP4_MUXER_CLOCK_FREQUENCIES)muxerConfig.clockfreq);
+        uint32_t clockFreq=s->getTimeBaseDen();
+        uint32_t nbTicks=s->getTimeBaseNum();
+        if(timescale)
+        {
+            if(clockFreq && nbTicks>1 && nbTicks<(1<<14))
+            {
+                if(!((nbTicks*timescale)%clockFreq))
+                    nbTicks=(nbTicks*timescale)/clockFreq;
+                else
+                    nbTicks=1;
+            }else
+            {
+                nbTicks=1;
+            }
+            clockFreq=timescale;
+        }
+        if(clockFreq && nbTicks)
+        {
+            c->time_base.den=clockFreq;
+            c->time_base.num=nbTicks;
+        }else
+            rescaleFps(s->getAvgFps1000(),&(c->time_base));
         myTimeBase=video_st->time_base=c->time_base;
         ADM_info("Video stream time base :%d,%d\n",video_st->time_base.num,video_st->time_base.den);
         c->gop_size=15;
@@ -196,15 +264,25 @@ bool muxerMP4::open(const char *file, ADM_videoStream *s,uint32_t nbAudioTrack,A
             ADM_info("Setting rotation to %s degrees clockwise\n",angle);
             av_dict_set(&(video_st->metadata), "rotate", angle, 0);
         }
-        ADM_assert(avformat_write_header(oc, &dict) >= 0);
+        //ADM_assert(avformat_write_header(oc, &dict) >= 0);
+        er = avformat_write_header(oc, &dict);
+        if(er < 0)
+        {
+            char str[AV_ERROR_MAX_STRING_SIZE]={0};
+            av_make_error_string(str, AV_ERROR_MAX_STRING_SIZE, er);
+            ADM_error("Writing header failed with error %d (%s)\n", er, str);
+            av_dict_free(&dict);
+            avio_close(oc->pb);
+            return false;
+        }
 
         ADM_info("Timebase codec = %d/%d\n",c->time_base.num,c->time_base.den);
         ADM_info("Timebase stream = %d/%d\n",video_st->time_base.num,video_st->time_base.den);
-        // The reason to enforce strict CFR is unclear. Disable rounding, which may result in collisions, for now.
-        if(myTimeBase.den==video_st->time_base.den && video_st->time_base.num==1)
+        // Rounding may result in timestamp collisions due to bad choice of timebase, handle with care.
+        if(myTimeBase.den>1 && myTimeBase.den==video_st->time_base.den && video_st->time_base.num==1)
         {
-            //roundup=myTimeBase.num;
-            ADM_info("Timebase roundup would have been %d, ignoring\n",myTimeBase.num);
+            roundup=myTimeBase.num;
+            ADM_info("Using %d as timebase roundup.\n",myTimeBase.num);
         }
         av_dict_free(&dict);
         vStream=s;
