@@ -51,20 +51,46 @@ ADM_ffNvEncEncoderHEVC::ADM_ffNvEncEncoderHEVC(ADM_coreVideoFilter *src,bool glo
 */
 bool ADM_ffNvEncEncoderHEVC::configureContext(void)
 {
-    switch(NvEncSettings.preset)
+    _context->bit_rate = -1;
+    _context->rc_max_rate = -1;
+
+    _context->gop_size = NvEncSettings.gopsize;
+    _context->max_b_frames = NvEncSettings.bframes;
+
+    switch(NvEncSettings.rc_mode)
     {
-#define MIAOU(x,y) case NV_FF_PRESET_##x: ADM_assert(!av_opt_set(_context->priv_data,"preset",y, 0));break;  
-        
-     MIAOU(HP,"hp")   
-     MIAOU(BD,"bd")   
-     MIAOU(LL,"ll")   
-     MIAOU(LLHP,"llhp")   
-     MIAOU(LLHQ,"llhq")   
-     MIAOU(HQ,"hq")                
-default:break;
+        case NV_FF_RC_AUTO:
+            switch(NvEncSettings.preset)
+            {
+#define MIAOU(x,y) case NV_FF_PRESET_##x: ADM_assert(!av_opt_set(_context->priv_data,"preset",y, 0));break;
+                MIAOU(HP,"hp")
+                MIAOU(HQ,"hq")
+                MIAOU(BD,"bd")
+                MIAOU(LL,"ll")
+                MIAOU(LLHP,"llhp")
+                MIAOU(LLHQ,"llhq")
+                default:break;
+#undef MIAOU
+            }
+            break;
+        case NV_FF_RC_CONSTQP:
+            _context->qmin = _context->qmax = NvEncSettings.quality;
+            ADM_assert(!av_opt_set(_context->priv_data,"rc","constqp",0));
+            break;
+        case NV_FF_RC_CBR:
+            _context->bit_rate = _context->rc_max_rate = NvEncSettings.bitrate*1000;
+            ADM_assert(!av_opt_set(_context->priv_data,"rc","cbr",0));
+            break;
+        case NV_FF_RC_VBR:
+            _context->bit_rate=NvEncSettings.bitrate*1000;
+            _context->rc_max_rate=NvEncSettings.max_bitrate*1000;
+            ADM_assert(!av_opt_set(_context->priv_data,"rc","vbr",0));
+            break;
+        default:
+            ADM_warning("Unsupported mode %d\n",NvEncSettings.rc_mode);
+            break;
     }
-    _context->bit_rate=NvEncSettings.bitrate*1000;
-    _context->rc_max_rate=NvEncSettings.max_bitrate*1000;
+
 #ifdef USE_NV12    
     _context->pix_fmt=  AV_PIX_FMT_NV12;        
 #else    
@@ -177,17 +203,34 @@ link:
 
 */
 bool         ADM_ffNvEncEncoderHEVC::isDualPass(void)
-{   
+{
+    if(NvEncSettings.twopass)
+        return true;
+    if(NvEncSettings.rc_mode == NV_FF_RC_AUTO)
+    {
+#define EQL(desc) NvEncSettings.preset == NV_FF_PRESET_##desc
+        if(EQL(SLOW) || EQL(LL) || EQL(LLHP) || EQL(LLHQ))
+            return true;
+#undef EQL
+#define EQL(desc) NvEncSettings.rc_mode == NV_FF_RC_##desc
+    }else if(EQL(CBR_LOWDELAY_HQ) || EQL(CBR_HQ) || EQL(VBR_HQ))
+        return true;
+#undef EQL
     return false;
 }
 
 /**
-    \fn jpegConfigure
-    \brief UI configuration for jpeg encoder
+    \fn ffNvEncConfigure
+    \brief UI configuration dialog
 */
-
-bool         ffNvEncConfigure(void)
+bool ffNvEncConfigure(void)
 {
+diaMenuEntry meRcMode[]={
+  {NV_FF_RC_AUTO,QT_TRANSLATE_NOOP("ffnvenc","Controlled by Preset")},
+  {NV_FF_RC_CONSTQP,QT_TRANSLATE_NOOP("ffnvenc","Constant Quality")},
+  {NV_FF_RC_CBR,QT_TRANSLATE_NOOP("ffnvenc","Constant Bitrate")},
+  {NV_FF_RC_VBR,QT_TRANSLATE_NOOP("ffnvenc","Variable Bitrate")},
+};
 diaMenuEntry mePreset[]={ 
   {NV_FF_PRESET_HP,QT_TRANSLATE_NOOP("ffnvenc","Low Quality")},
   {NV_FF_PRESET_HQ,QT_TRANSLATE_NOOP("ffnvenc","High Quality")},
@@ -197,21 +240,53 @@ diaMenuEntry mePreset[]={
   {NV_FF_PRESET_LLHQ,QT_TRANSLATE_NOOP("ffnvenc","Low Latency (HQ)")}
 };
 
-        ffnvenc_encoder *conf=&NvEncSettings;
+    ffnvenc_encoder *conf=&NvEncSettings;
 
 #define PX(x) &(conf->x)
 
-        diaElemMenu      qzPreset(PX(preset),QT_TRANSLATE_NOOP("ffnvenc","Preset:"),6,mePreset);        
-        diaElemUInteger  bitrate(PX(bitrate),QT_TRANSLATE_NOOP("ffnvenc","Bitrate (kbps):"),1,50000);
-        diaElemUInteger  maxBitrate(PX(max_bitrate),QT_TRANSLATE_NOOP("ffnvenc","Max Bitrate (kbps):"),1,50000);
-          /* First Tab : encoding mode */
-        diaElem *diamode[]={&qzPreset,&bitrate,&maxBitrate};
+    diaElemMenu     rcmode(PX(rc_mode),QT_TRANSLATE_NOOP("ffnvenc","RC Mode:"),4,meRcMode);
+    diaElemMenu     qzPreset(PX(preset),QT_TRANSLATE_NOOP("ffnvenc","Preset:"),6,mePreset);
 
-        if( diaFactoryRun(QT_TRANSLATE_NOOP("ffnvenc","nvEnc HEVC  configuration"),3,diamode))
-        {
-          
-          return true;
-        }
-         return false;
+    diaElemUInteger qual(PX(quality),QT_TRANSLATE_NOOP("ffnvenc","Quality:"),0,51);
+    diaElemUInteger bitrate(PX(bitrate),QT_TRANSLATE_NOOP("ffnvenc","Bitrate (kbps):"),1,800000);
+    diaElemUInteger maxBitrate(PX(max_bitrate),QT_TRANSLATE_NOOP("ffnvenc","Max Bitrate (kbps):"),1,800000);
+    diaElemUInteger gopSize(PX(gopsize),QT_TRANSLATE_NOOP("ffnvenc","GOP Size:"),8,250);
+    diaElemUInteger maxBframes(PX(bframes),QT_TRANSLATE_NOOP("ffnvenc","Maximum Consecutive B-Frames:"),0,4);
+
+    diaElemToggle   dualPass(PX(twopass),QT_TRANSLATE_NOOP("ffnvenc","2-Pass Mode"));
+    diaElemReadOnlyText hint(QT_TRANSLATE_NOOP("ffnvenc","Low Latency presets always use 2-pass mode"),NULL);
+
+    diaElemFrame    rateControl(QT_TRANSLATE_NOOP("ffnvenc","Rate Control"));
+    diaElemFrame    frameControl(QT_TRANSLATE_NOOP("ffnvenc","Frame Control"));
+
+    rateControl.swallow(&rcmode);
+    rateControl.swallow(&qzPreset);
+    rateControl.swallow(&qual);
+    rateControl.swallow(&bitrate);
+    rateControl.swallow(&maxBitrate);
+    rateControl.swallow(&dualPass);
+    rateControl.swallow(&hint);
+
+    rcmode.link(meRcMode,  1,&qzPreset);
+    rcmode.link(meRcMode+1,1,&qual);
+    rcmode.link(meRcMode+1,1,&dualPass);
+    rcmode.link(meRcMode+2,1,&bitrate);
+    rcmode.link(meRcMode+2,1,&dualPass);
+    rcmode.link(meRcMode+3,1,&bitrate);
+    rcmode.link(meRcMode+3,1,&maxBitrate);
+    rcmode.link(meRcMode+3,1,&dualPass);
+
+    qzPreset.link(mePreset,  1,&dualPass);
+    qzPreset.link(mePreset+1,1,&dualPass);
+    qzPreset.link(mePreset+2,1,&dualPass);
+
+    frameControl.swallow(&gopSize);
+    frameControl.swallow(&maxBframes);
+
+    diaElem *diamode[]={&rateControl,&frameControl};
+
+    if(diaFactoryRun(QT_TRANSLATE_NOOP("ffnvenc","NVENC HEVC configuration"),2,diamode))
+        return true;
+    return false;
 }
 // EOF
