@@ -20,6 +20,8 @@
 #include "ADM_jpegEncoder.h"
 #include "DIA_factory.h"
 
+
+
 jpeg_encoder jpegConf= JPEG_CONF_DEFAULT;
 /**
         \fn ADM_jpegEncoder
@@ -29,12 +31,22 @@ ADM_jpegEncoder::ADM_jpegEncoder(ADM_coreVideoFilter *src,bool globalHeader) : A
     printf("[jpegEncoder] Creating.\n");
     targetColorSpace=(ADM_colorspace)jpegConf.colorSpace;
 }
+
+/**
+    \fn configureContext
+*/
+bool ADM_jpegEncoder::configureContext(void)
+{
+    _context->flags |= AV_CODEC_FLAG_QSCALE;
+    return true;
+}
+
 /**
     \fn setup
 */
 bool ADM_jpegEncoder::setup(void)
 {
-    return ADM_coreVideoEncoderFFmpeg::setup(CODEC_ID_MJPEG);
+    return ADM_coreVideoEncoderFFmpeg::setup(AV_CODEC_ID_MJPEG);
 }
 
 
@@ -53,14 +65,16 @@ ADM_jpegEncoder::~ADM_jpegEncoder()
 bool         ADM_jpegEncoder::encode (ADMBitstream * out)
 {
     if(false==preEncode()) return false;
-    _context->flags |= CODEC_FLAG_QSCALE;
-    _frame->quality = (int) floor (FF_QP2LAMBDA * jpegConf.quantizer+ 0.5);
-    int sz=0;
-    if ((sz = avcodec_encode_video (_context, out->data, out->bufferSize, _frame)) < 0)
+    _frame->quality = (int)jpegConf.quantizer * FF_QP2LAMBDA;
+
+    int sz=0,r,gotData;
+    r=encodeWrapper(_frame,out);
+    if(r<0)
     {
-        ADM_error("[jpeg] Error %d encoding video\n",sz);
+        ADM_warning("[ffHuff] Error %d encoding video\n",r);
         return false;
     }
+    sz=r;    
     
     out->len=sz;
     out->pts=out->dts=image->Pts;
