@@ -53,8 +53,8 @@ DECLARE_VIDEO_FILTER(   CropFilter,   // Class
                         ADM_UI_TYPE_BUILD,         // UI
                         VF_TRANSFORM,            // Category
                         "crop",            // internal name (must be uniq!)
-                        QT_TRANSLATE_NOOP("crop","crop"),            // Display name
-                        QT_TRANSLATE_NOOP("crop","crop filter") // Description
+                        QT_TRANSLATE_NOOP("crop","Crop"),            // Display name
+                        QT_TRANSLATE_NOOP("crop","Crop filter") // Description
                     );
 
 
@@ -73,6 +73,7 @@ CropFilter::CropFilter(ADM_coreVideoFilter *in,CONFcouple *couples) :ADM_coreVid
             configuration.bottom=0;
             configuration.left=0;
             configuration.right=0;
+            configuration.rubber_is_hidden=false;
         }
         if(  in->getInfo()->width<(configuration.right+configuration.left))
                 {
@@ -169,10 +170,10 @@ const char *CropFilter::getConfiguration(void)
 {
     static char conf[80];
     conf[0]=0;
-    snprintf(conf,80,"Crop : %"PRIu32"x%"PRIu32" => %"PRIu32"x%"PRIu32,
-                previousFilter->getInfo()->width,
-                previousFilter->getInfo()->height,
-                info.width,info.height
+    snprintf(conf,80,"Crop : %dx%d => %dx%d",
+                (int)previousFilter->getInfo()->width,
+                (int)previousFilter->getInfo()->height,
+                (int)info.width,(int)info.height
                 );
     return conf;
 }
