@@ -102,18 +102,18 @@ enum YADIFDeint {
 class yadifFilter : public  ADM_coreVideoFilterCached
 {
 protected:
-                    ADMImage    *original;
                     yadif       configuration;
                     void        updateInfo(void);
 public:
                     yadifFilter(ADM_coreVideoFilter *previous,CONFcouple *conf);
-                    ~yadifFilter();
+                    ~yadifFilter() {};
 
         virtual const char   *getConfiguration(void);                 /// Return  current configuration as a human readable string
         virtual bool         getNextFrame(uint32_t *fn,ADMImage *image);           /// Return the next image
         virtual bool         getCoupledConf(CONFcouple **couples) ;   /// Return the current filter configuration
 		virtual void setCoupledConf(CONFcouple *couples);
         virtual bool         configure(void) ;                        /// Start graphical user interface
+        virtual bool         goToTime(uint64_t usSeek);
         
 protected:
     void (*filter_line) (uint8_t *dst, const uint8_t  *prev, const uint8_t  *cur, const uint8_t  *next, int w, int prefs, int mrefs, int parity, int mode);
@@ -134,8 +134,6 @@ DECLARE_VIDEO_FILTER(   yadifFilter,   // Class
                     );
 
 //
-static void filter_plane(int mode, uint8_t *dst, int dst_stride, const uint8_t *prev0, const uint8_t *cur0, const uint8_t *next0, int refs, int w, int h, int parity, int tff, int mmx);
-
 
 /**
     \fn constructor
@@ -143,13 +141,12 @@ static void filter_plane(int mode, uint8_t *dst, int dst_stride, const uint8_t *
 yadifFilter::yadifFilter(ADM_coreVideoFilter *in, CONFcouple *setup): 
             ADM_coreVideoFilterCached(10,in,setup)
 {
-    original=new ADMImageDefault(in->getInfo()->width,in->getInfo()->height);
     if(!setup || !ADM_paramLoad(setup,yadif_param,&configuration))
     {
         // Default value
         configuration.mode=YADIF_MODE_SEND_FRAME;
         configuration.deint=YADIF_DEINT_ALL; 
-        configuration.parity=YADIF_PARITY_AUTO;
+        configuration.parity=YADIF_PARITY_TFF;
     }
     updateInfo();
 
@@ -158,7 +155,7 @@ yadifFilter::yadifFilter(ADM_coreVideoFilter *in, CONFcouple *setup):
     filter_end=filter_end_c;
 
 #ifdef CAN_DO_INLINE_X86_ASM    
-     if (CpuCaps::hasSSE3()) 
+     if (CpuCaps::hasSSSE3())
      {
         filter_line=adm_yadif_filter_line_ssse3;
      }else 
@@ -170,28 +167,26 @@ yadifFilter::yadifFilter(ADM_coreVideoFilter *in, CONFcouple *setup):
     
     myName="yadif";
 }
-/**
-    \fn destructor
-*/
-yadifFilter::~yadifFilter()
-{
-        delete  original;
-        original=NULL;
-}
 /**
     \fn updateInfo
 */
-
 void yadifFilter::updateInfo(void)
 {
-  memcpy(&info,previousFilter->getInfo(),sizeof(info)); 
-  if(configuration.mode &1 ) // Bob
-  {
-    info.frameIncrement/=2;
-  }
+    memcpy(&info,previousFilter->getInfo(),sizeof(info));
+    if(configuration.mode &1 ) // Bob
+    {
+        info.frameIncrement/=2;
+        if(info.timeBaseNum && info.timeBaseDen)
+        {
+            if(info.timeBaseDen<=30000 && (info.timeBaseNum & 1))
+                info.timeBaseDen*=2;
+            else
+                info.timeBaseNum/=2;
+        }
+    }
 }
 /**
-    \fn updateInfo
+    \fn configure
 */
 bool yadifFilter::configure( void) 
 {
@@ -255,5 +250,18 @@ const char *yadifFilter::getConfiguration(void)
     return conf;
 }
 
+/**
+    \fn goToTime
+    \brief Seek in filter preview mode
+*/
+bool yadifFilter::goToTime(uint64_t usSeek)
+{
+    uint32_t oldFrameIncrement=info.frameIncrement;
+    if(configuration.mode &1) // Bob
+        info.frameIncrement*=2;
+    bool r=ADM_coreVideoFilterCached::goToTime(usSeek);
+    info.frameIncrement=oldFrameIncrement;
+    return r;
+}
 
 #include "ADM_vidYadif_body.cpp"
